from datetime import timedelta

from django.db import models
from django.utils import timezone


class SerialHistory(models.Model):
    serial_key = models.CharField(max_length=20, unique=True)
    created_at = models.DateTimeField(auto_now_add=True)
    status = models.CharField(max_length=20, default="active")
    total_days = models.IntegerField(default=30)
    days_active = models.IntegerField(default=0)
    suspended_at = models.DateTimeField(null=True, blank=True)
    expired_date = models.DateTimeField(null=True, blank=True)

    def __str__(self):
        return self.serial_key

    def update_remaining_days(self):
        """
        Updates the number of remaining days based on the days of activity and the total number of days.
        """
        if self.status == "active":
            elapsed_days = (timezone.now() - self.created_at).days
            self.days_active += elapsed_days
        self.remaining_days = self.total_days - self.days_active
        self.expired_date = self.created_at + timedelta(days=self.total_days)
        self.save()

    def resume(self):
        """
        Resumes the validity of the serial key by updating the suspension dates and returning its status to active.
        """
        if self.status == "deactivated" and self.suspended_at:
            suspended_duration = (timezone.now() - self.suspended_at).days
            self.days_active -= suspended_duration
            self.status = "active"
            self.suspended_at = None
            self.save()


class Server(models.Model):
    server_id = models.CharField(max_length=2, unique=True)
    name = models.CharField(max_length=100)
    ping = models.IntegerField(default=-1)
    selected = models.BooleanField(default=False)

    def __str__(self):
        return f"{self.server_id} - {self.name}"


class Config(models.Model):
    ip = models.GenericIPAddressField()
    location = models.CharField(max_length=100)
    dns = models.GenericIPAddressField()
    gateway = models.GenericIPAddressField()
    selected_server = models.ForeignKey(
        Server, on_delete=models.SET_NULL, null=True, blank=True
    )
    is_activated = models.BooleanField(default=False)
    is_connected = models.BooleanField(default=False)

    def __str__(self):
        return f"Config - {self.location}"


class WiFiStatus(models.Model):
    band = models.CharField(max_length=10)  # "2.4GHz", "5GHz", "6GHz"
    frequency = models.CharField(max_length=10)  # Частота, например, "2412 MHz"
    transmit_power = models.FloatField()  # Мощность передатчика в dBm
    enabled = models.BooleanField(default=False)


class WorkProfile(models.Model):
    PROFILE_TYPE_CHOICES = [
        ("no_restriction", "No Restriction"),
        ("selection_mode", "Selection Mode"),
        ("exclusion_mode", "Exclusion Mode"),
    ]

    name = models.CharField(max_length=100, unique=True)
    profile_type = models.CharField(
        max_length=20, choices=PROFILE_TYPE_CHOICES, default="no_restriction"
    )
    is_active = models.BooleanField(default=True)
    selected_urls = models.ManyToManyField(
        "ProfileURL", blank=True, related_name="work_profiles"
    )
    selected_apps = models.ManyToManyField(
        "Application", blank=True, related_name="work_profiles"
    )

    def __str__(self):
        return self.name


class ProfileURL(models.Model):
    url = models.CharField(max_length=255, unique=True)

    def __str__(self):
        return self.url


class Application(models.Model):
    name = models.CharField(max_length=100, unique=True)

    def __str__(self):
        return self.name
