import subprocess
import logging
from django.db import transaction
from django.utils import timezone
from typing import List, Dict, Any, Tuple, Optional
import re

from .models import Config, Server, SerialHistory, WiFiStatus
from .scheme import HostapdConfig


logger = logging.getLogger(__name__)


# NOTE нужно добавить обработку ошибки, когда закончился период серийного номера
"""(base) goodman@goodman-hp:/opt/kelvpn/bin$ sudo ./KelVPNCli -C 04
[sudo] пароль для goodman: 
Cmd::Connect: (QVariant(int, 4))
Connection state updated: Disconnected
Activating tasks...
Cmd::Connect: Task activated
Connection state updated: Connecting
Task::Connect: Connecting to server: Europe.Netherlands.5...
Error: Serial key expired
Task::Connect: Connection error signal caught! Abort.
Task::Connect: Connection failed signal caught! Abort.
Connection state updated: Disconnected
All tasks are finished.
(base) goodman@goodman-hp:/opt """


class KelVPNCliService:
    def __init__(self) -> None:
        self.cli_path: str = "/opt/kelvpn/bin/KelVPNCli"

    def execute_command(self, args: List[str], timeout: int = 120) -> str:
        """
        Executes a command with the provided arguments and returns the result.

        :param args: List of command arguments.
        :param timeout: Timeout for the command execution in seconds.
        :return: Standard output from the command.
        :raises Exception: If the command execution fails or times out.
        """
        try:
            result = subprocess.run(
                args, capture_output=True, text=True, check=True, timeout=timeout
            )
            return result.stdout
        except subprocess.CalledProcessError as e:
            logger.error(f"Command execution error: {e.stderr}")
            raise Exception(f"Command execution error: {e.stderr}") from e
        except subprocess.TimeoutExpired:
            logger.error("Command execution time exceeded the limit.")
            raise Exception("Command execution time exceeded the limit.") from None

    def start_service(self, server_id: str) -> Tuple[str, bool]:
        """
        Starts KelVPNCli with the selected server.

        :param server_id: The ID of the server to connect to.
        :return: Tuple containing the command output and a success flag.
        """
        cmd_connect: List[str] = [self.cli_path, "-C", server_id]
        output: str = self.execute_command(cmd_connect)

        if (
            "Connection state updated: Connected" not in output
            and "Task::Connect: Success!" not in output
        ):
            self.update_config(is_connected=False)
            logger.error(f"Failed to establish a connection. Command output: {output}")
            return output, False

        self.update_config(is_connected=True, is_activated=True)
        return output, True

    def disconnect_service(self) -> str:
        """
        Disconnects KelVPNCli and stops related services.

        :return: Combined output from the commands as a string.
        """
        try:
            # Disconnect KelVPNCli
            cmd_disconnect: List[str] = [self.cli_path, "-D"]
            disconnect_output: str = self.execute_command(cmd_disconnect)
            logger.info(f"KelVPNCli disconnected: {disconnect_output}")

            # Stop KelVPNCheck.service
            cmd_stop_check: List[str] = ["systemctl", "stop", "KelVPNCheck.service"]
            check_service_output: str = self.execute_command(cmd_stop_check)
            logger.info(f"KelVPNCheck.service stopped: {check_service_output}")

            # Stop KelVPNService.service
            cmd_stop_service: List[str] = ["systemctl", "stop", "KelVPNService.service"]
            service_output: str = self.execute_command(cmd_stop_service)
            logger.info(f"KelVPNService.service stopped: {service_output}")

            # Update configuration
            self.update_config(is_connected=False)

            # Combine all outputs into a single string
            combined_output = (
                f"{disconnect_output}\n{check_service_output}\n{service_output}"
            )
            return combined_output
        except Exception as e:
            logger.error(f"Error disconnecting service: {str(e)}")
            return str(e)

    def restart_service(self) -> Tuple[str, bool]:
        """
        Restarts the KelVPNService and kelvpn_backend.service.

        :return: Tuple containing the output from the restart commands and success flag.
        """
        try:
            # Update configuration
            self.update_config(is_connected=False)

            # Restart KelVPNService
            cmd_restart_kelvpn: List[str] = ["systemctl", "restart", "KelVPNService.service"]
            kelvpn_output: str = self.execute_command(cmd_restart_kelvpn)
            logger.info(f"KelVPNService restarted successfully: {kelvpn_output}")

            # Restart kelvpn_backend.service
            cmd_restart_backend: List[str] = ["systemctl", "restart", "kelvpn_backend.service"]
            backend_output: str = self.execute_command(cmd_restart_backend)
            logger.info(f"kelvpn_backend.service restarted successfully: {backend_output}")

            return kelvpn_output + "\n" + backend_output, True
        except Exception as e:
            logger.error(f"Error restarting services: {str(e)}")
            return str(e), False

    def reboot_system(self) -> str:
        """
        Reboots the system

        :return: Standard output from the reboot command.
        """
        cmd_reboot: List[str] = ["reboot"]
        output: str = self.execute_command(cmd_reboot)
        logger.info("System reboot initiated.")
        return output

    def get_service_status(self) -> str:
        """
        Retrieves the status of the KelVPNService.

        :return: Standard output from the status command.
        """
        cmd_status: List[str] = [
            "systemctl",
            "status",
            "KelVPNService",
            "--no-pager",
        ]
        output: str = self.execute_command(cmd_status)
        return output

    def set_serial(self, serial_key: str) -> Tuple[str, bool, str]:
        """
        Sets the serial key using the best server and starts KelVPN.

        :param serial_key: The serial key to activate.
        :return: Tuple containing the command output, success flag, and a message indicating the result.
        """
        servers_ping: List[Dict[str, Any]] = self.get_server_pings()

        best_server = min(servers_ping, key=lambda server: server["ping"])
        logger.info(f"best_server {best_server} for activation.")
        if best_server:
            server_id = best_server["server_id"]
            logger.info(f"Using best server with ID {server_id} for activation.")
        else:
            raise Exception("No server available for activation.")

        cmd_set_serial: List[str] = [
            self.cli_path,
            "-C",
            server_id,
            "-S",
            serial_key,
        ]
        output: str = self.execute_command(cmd_set_serial)
        logger.debug(f"Output: {output}")
        success, message = self._parse_set_serial_output(output)
        if not success:
            return output, success, message

        # Создаем или получаем существующую запись SerialHistory
        serial, created = SerialHistory.objects.get_or_create(serial_key=serial_key, defaults={
            "status": "active",
            "total_days": 30,
            "days_active": 0,
        })

        if created:
            logger.info(f"New serial key {serial.serial_key} created with default values.")
        else:
            serial.update_remaining_days()
            if serial.status != "active":
                serial.status = "active"
                serial.suspended_at = None
                serial.save()
                logger.info(f"Serial key {serial.serial_key} activated successfully.")
            if serial.status == "deactivated":
                serial.resume()
                logger.info(f"Serial key {serial.serial_key} resumed from suspension.")

        return output, success, message

    def _parse_set_serial_output(self, output: str) -> Tuple[bool, str]:
        """
        Parses the output of the set_serial command to determine success and returns a message.

        :param output: The standard output from the set_serial command.
        :return: Tuple of boolean success flag and message.
        """
        config: Optional[Config] = Config.objects.first()
        if not config:
            logger.error("Configuration not found.")
            raise Exception("Configuration not found.")

        if "Task::Serial: Serial key is set" in output:
            if "Connection state updated: Connected" in output:
                config.is_activated = True
                config.is_connected = True
                config.save()
                logger.info("Serial key successfully activated and connected.")
                return (
                    True,
                    "Serial key successfully activated and connection established.",
                )
            elif "Task::Connect: Unable to finish task in time" in output:
                config.is_activated = True
                config.is_connected = False
                config.save()
                logger.warning(
                    "Serial key activated, but connection to the server failed."
                )
                return (
                    True,
                    "Serial key activated, but connection to the server failed.",
                )

        if "Task::Serial: Error:" in output:
            if "Connection state updated: Connected" not in output:
                logger.error(f"Serial key error without connection: {output}")
                return (
                    False,
                    "Serial key error occurred, and connection was not established.",
                )
            if (
                "Cmd::Serial: Invalid serial key format" in output
                and "Connection state updated: Connected" in output
            ):
                logger.error(f"Invalid serial key format: {output}")
                return True, "The serial key format is invalid."
            if (
                "Cmd::Serial: Invalid serial key format" in output
                and "Connection state updated: Connected" not in output
            ):
                logger.error(f"Invalid serial key format: {output}")
                return False, "The serial key format is invalid."

        if "Connection state updated: Connected" in output:
            config.is_activated = True
            config.is_connected = True
            config.save()
            logger.info("Service connected successfully.")
            return True, "Service connected successfully."

        logger.error(f"Unexpected output: {output}")
        return False, "Unexpected error occurred. Check logs for details."

    def get_serial_history(self) -> List[Dict[str, Any]]:
        """
        Retrieves and stores the serial history.

        :return: List of dictionaries containing serial keys and their creation timestamps.
        """
        cmd_history: List[str] = [self.cli_path, "-H"]
        output: str = self.execute_command(cmd_history)
        serial_keys: List[str] = self._parse_serial_history(output)

        with transaction.atomic():
            for key in serial_keys:
                try:
                    obj, created = SerialHistory.objects.get_or_create(serial_key=key, defaults={"status": "deactivated"})
                    if created:
                        logger.info(f"Created new SerialHistory for serial key {key}.")
                    else:
                        logger.warning(f"Serial key {key} already exists.")
                except Exception as e:
                    logger.error(f"Error while creating or getting SerialHistory for serial key {key}. Error: {str(e)}")
                    pass

        history: List[SerialHistory] = SerialHistory.objects.all().order_by(
            "-created_at"
        )
        
        for entry in history:
            entry.update_remaining_days()
        logger.info(f"history: {list(history)}")
        return [
            {
                "serial_key": entry.serial_key,
                "created_at": entry.created_at,
                "status": entry.status,
                "total_days": entry.total_days,
                "days_active": entry.days_active,
                "remaining_days": entry.total_days - entry.days_active,
                "expired_date": entry.expired_date,
            }
            for entry in history
        ]

    def _parse_serial_history(self, output: str) -> List[str]:
        """
        Parses the output of the KelVPN -H command and returns a list of serial keys.

        :param output: The standard output from the KelVPNCli -H command.
        :return: List of serial keys.
        """
        serial_keys: List[str] = []
        lines: List[str] = output.splitlines()

        # Find the line with "Serial History size"
        size_line: Optional[str] = next(
            (line for line in lines if "Serial History size" in line), None
        )
        if size_line:
            try:
                size: int = int(
                    re.search(r"Serial History size: (\d+)", size_line).group(1)
                )
            except (AttributeError, ValueError):
                size = 0
        else:
            size = 0

        for line in lines:
            match = re.match(r"\d{2} : ([A-Z0-9-]{19})", line)
            if match:
                serial_keys.append(match.group(1))

        return serial_keys

    def deactivate_serial(self) -> Tuple[str, bool, Optional[SerialHistory]]:
        """
        Deactivates the serial key by resetting it using the KelVPNCli and updates the status and suspension date.

        :return: Tuple containing the command output, success flag, and the deactivated serial key object.
        """
        cmd_reset_serial: List[str] = [self.cli_path, "-W"]
        output: str = self.execute_command(cmd_reset_serial)

        if "Task::ResetSerial: Success!" in output:
            active_serial = SerialHistory.objects.filter(status="active").first()
            if active_serial:
                active_serial.status = "deactivated"
                active_serial.suspended_at = timezone.now()
                active_serial.save()

            self.update_config(is_connected=False, is_activated=False)
            logger.info(
                f"Serial key {active_serial.serial_key if active_serial else 'unknown'} deactivated successfully."
            )
            return output, True, active_serial

        logger.error(f"Failed to deactivate serial key. Command output: {output}")
        return output, False, None

    def get_server_pings(self) -> List[Dict[str, Any]]:
        """
        Retrieves the list of servers with their pings and updates the database.

        :return: List of dictionaries containing server details.
        """
        cmd_servers: List[str] = [self.cli_path, "-L", "-R"]
        output: str = self.execute_command(cmd_servers)
        servers_ping: List[Dict[str, Any]] = self._parse_server_ping_list(output)

        with transaction.atomic():
            for server in servers_ping:
                Server.objects.update_or_create(
                    server_id=server["id"],
                    defaults={
                        "name": server["name"],
                        "ping": server["ping"],
                        "selected": False,
                    },
                )

        config, created = Config.objects.get_or_create(
            id=1,
            defaults={
                "ip": "0.0.0.0",
                "location": "",
                "dns": "8.8.8.8",
                "gateway": "192.168.100.1",
                "selected_server": None,
                "is_connected": False,
                "is_activated": False,
            },
        )

        logger.debug(f"Config: {config}")
        logger.debug(f"Selected server in config: {config.selected_server}")

        if config.selected_server:
            logger.debug(f"Using server from config: {config.selected_server}")

            Server.objects.filter(selected=True).update(selected=False)
            selected_server = Server.objects.filter(
                server_id=config.selected_server.server_id
            ).first()

            if selected_server:
                selected_server.selected = True
                selected_server.save()
            else:
                logger.debug(
                    f"Warning: Server from config not found in the current ping list."
                )
        else:
            logger.debug("No server is selected, selecting the best server...")

            Server.objects.filter(selected=True).update(selected=False)

            best_server: Optional[Server] = (
                Server.objects.filter(ping__gte=0).order_by("ping").first()
            )

            if best_server:
                config.selected_server = best_server
                best_server.selected = True
                best_server.save()
                config.save()
                logger.debug(
                    f"Automatically selected server: {best_server} (selected: {best_server.selected})"
                )
            else:
                logger.debug("No suitable server found to select.")

        return [
            {
                "server_id": server.server_id,
                "name": server.name,
                "ping": server.ping,
                "selected": server.selected,
            }
            for server in Server.objects.all().order_by("ping")
        ]

    def _parse_server_ping_list(self, output: str) -> List[Dict[str, Any]]:
        """
        Parses the output of the KelVPNCli -L -R command and returns a list of servers with pings.

        :param output: The standard output from the KelVPNCli -L -R command.
        :return: List of dictionaries containing server details.
        """
        servers_ping: List[Dict[str, Any]] = []
        lines: List[str] = output.splitlines()

        # Regular expression to match server lines with pings
        server_ping_line_regex = re.compile(
            r"\|\s*(\d{2})\s*\|\s*(\d{1,4})\s*\|\s*([^\|]+)\s*\|"
        )

        for line in lines:
            match = server_ping_line_regex.match(line)
            if match:
                server_id: str = match.group(1).strip()
                ping_str: str = match.group(2).strip()
                server_name: str = match.group(3).strip()

                try:
                    ping: int = int(ping_str)
                except ValueError:
                    ping = -1

                if server_id and server_name:
                    servers_ping.append(
                        {
                            "id": server_id,
                            "ping": ping,
                            "name": server_name,
                            "selected": False,
                        }
                    )
                    logger.debug(
                        f"Parsed server with ping: ID={server_id}, Ping={ping}, Name={server_name}"
                    )

        logger.info(f"Total servers with ping parsed: {len(servers_ping)}")
        return servers_ping

    def update_config(
        self, is_connected: bool, is_activated: Optional[bool] = None
    ) -> None:
        """
        Updates the config's connection and activation status.

        :param is_connected: Boolean indicating if the service is connected.
        :param is_activated: Optional boolean indicating if the service is activated.
        """
        config: Optional[Config] = Config.objects.first()
        if config:
            config.is_connected = is_connected
            if is_activated is not None:
                config.is_activated = is_activated
            config.save()

    def _get_ip_addresses(self):
        # result = subprocess.run(["nmcli", "device", "show"], capture_output=True, text=True)
        # addresses = re.findall(r"IP4\.ADDRESS\[\d+\]:\s+([\d.]+)", result.stdout)
        try:
            external_ip = subprocess.check_output(["curl", "-s", "ifconfig.me"]).decode().strip()
            # logger.debug(f"ip {external_ip}")
            return external_ip

        except subprocess.CalledProcessError as e:
            logger.debug(f"Error retrieving external IP: {e}")
            return None


    def _get_gateway(self):
        """
        Получает IP-адрес шлюза с приоритетом на tun0 (VPN), затем eth0 (Ethernet) и, наконец, wlan0 (Wi-Fi).
        """
        result = subprocess.run(
            ["nmcli", "device", "show"], capture_output=True, text=True
        )
        output = result.stdout

        # Определяем приоритет интерфейсов
        priority_interfaces = ["tun0", "eth0", "wlan0"]

        for interface in priority_interfaces:
            if f"GENERAL.DEVICE:                         {interface}" in output:
                match = re.search(
                    rf"{interface}\n(?:.*\n)*?IP4\.GATEWAY:\s+([\d.]+)", output
                )
                if match:
                    return match.group(1)

        return "-"

    def _get_dns_servers(self):
        dns_servers = []
        try:
            with open('/etc/resolv.conf', 'r') as file:
                for line in file:
                    if line.startswith("nameserver"):
                        dns_servers.append(line.split()[1])
        except Exception as e:
            logger.error(f"Error retrieving DNS servers from resolv.conf: {str(e)}")
        return dns_servers

    def update_network_config(self):
        ip_addresses = self._get_ip_addresses()
        gateway = self._get_gateway()
        dns_servers = self._get_dns_servers()
        logger.info(f"ip {ip_addresses}, gateway {gateway}, dns_servers {dns_servers}")
        config, created = Config.objects.get_or_create(id=1)
        config.ip = ip_addresses if ip_addresses else "-"
        config.gateway = gateway
        config.dns = ", ".join(dns_servers)
        
        config.save()
        logger.info("Network configuration updated successfully.")


class WifiManager:
    CONFIG_PATH = "/etc/hostapd/hostapd.conf"

    @staticmethod
    def start_hostapd():
        """
        Запускает hostapd.
        """
        subprocess.run(["systemctl", "start", "hostapd"])

    @staticmethod
    def stop_hostapd():
        """
        Останавливает hostapd.
        """
        subprocess.run(["systemctl", "stop", "hostapd"])

    @staticmethod
    def toggle_hostapd(enable: bool):
        """
        Включает или отключает hostapd в зависимости от параметра enable.
        """
        if enable:
            WifiManager.start_hostapd()
        else:
            WifiManager.stop_hostapd()

    @staticmethod
    def read_config() -> HostapdConfig:
        config = {}
        WifiManager.update_wifi_status()
        with open(WifiManager.CONFIG_PATH, "r") as file:
            for line in file:
                line = line.strip()
                if line.startswith("interface="):
                    config["interface"] = line.split("=")[1]
                elif line.startswith("driver="):
                    config["driver"] = line.split("=")[1]
                elif line.startswith("ssid="):
                    config["ssid"] = line.split("=")[1]
                elif line.startswith("hw_mode="):
                    config["hw_mode"] = line.split("=")[1]
                elif line.startswith("channel="):
                    config["channel"] = int(line.split("=")[1])
                elif line.startswith("wmm_enabled="):
                    config["wmm_enabled"] = line.split("=")[1] == "1"
                elif line.startswith("macaddr_acl="):
                    config["macaddr_acl"] = int(line.split("=")[1])
                elif line.startswith("auth_algs="):
                    config["auth_algs"] = int(line.split("=")[1])
                elif line.startswith("ignore_broadcast_ssid="):
                    config["ignore_broadcast_ssid"] = line.split("=")[1] == "1"
                elif line.startswith("wpa="):
                    config["wpa"] = int(line.split("=")[1])
                elif line.startswith("wpa_passphrase="):
                    config["wpa_passphrase"] = line.split("=")[1]
                elif line.startswith("wpa_key_mgmt="):
                    config["wpa_key_mgmt"] = line.split("=")[1]
                elif line.startswith("rsn_pairwise="):
                    config["rsn_pairwise"] = line.split("=")[1]
                elif line.startswith("network_mode="):
                    config["network_mode"] = line.split("=")[1]
                elif line.startswith("enabled="):
                    config["enabled"] = line.split("=")[1] == "1"

        # Определяем частоту на основе hw_mode и channel
        if config.get("hw_mode") == "g" and 1 <= config.get("channel", 0) <= 14:
            config["frequency"] = "2.4GHz"
        elif config.get("hw_mode") == "a" and 36 <= config.get("channel", 0) <= 165:
            config["frequency"] = "5GHz"
        elif config.get("hw_mode") == "6" and 1 <= config.get("channel", 0) <= 233:
            config["frequency"] = "6GHz"

        return HostapdConfig(**config)

    @staticmethod
    def update_config(config: HostapdConfig):
        with open(WifiManager.CONFIG_PATH, "r") as file:
            lines = file.readlines()

        with open(WifiManager.CONFIG_PATH, "w") as file:
            for line in lines:
                line = line.strip()
                if config.interface and line.startswith("interface="):
                    file.write(f"interface={config.interface}\n")
                elif config.driver and line.startswith("driver="):
                    file.write(f"driver={config.driver}\n")
                elif config.ssid and line.startswith("ssid="):
                    file.write(f"ssid={config.ssid}\n")
                elif config.hw_mode and line.startswith("hw_mode="):
                    file.write(f"hw_mode={config.hw_mode}\n")
                elif config.channel and line.startswith("channel="):
                    file.write(f"channel={config.channel}\n")
                elif config.wmm_enabled is not None and line.startswith("wmm_enabled="):
                    file.write(f"wmm_enabled={'1' if config.wmm_enabled else '0'}\n")
                elif config.macaddr_acl and line.startswith("macaddr_acl="):
                    file.write(f"macaddr_acl={config.macaddr_acl}\n")
                elif config.auth_algs and line.startswith("auth_algs="):
                    file.write(f"auth_algs={config.auth_algs}\n")
                elif config.ignore_broadcast_ssid is not None and line.startswith(
                    "ignore_broadcast_ssid="
                ):
                    file.write(
                        f"ignore_broadcast_ssid={'1' if config.ignore_broadcast_ssid else '0'}\n"
                    )
                elif config.wpa and line.startswith("wpa="):
                    file.write(f"wpa={config.wpa}\n")
                elif config.wpa_passphrase and line.startswith("wpa_passphrase="):
                    file.write(f"wpa_passphrase={config.wpa_passphrase}\n")
                elif config.wpa_key_mgmt and line.startswith("wpa_key_mgmt="):
                    file.write(f"wpa_key_mgmt={config.wpa_key_mgmt}\n")
                elif config.rsn_pairwise and line.startswith("rsn_pairwise="):
                    file.write(f"rsn_pairwise={config.rsn_pairwise}\n")
                elif config.network_mode and line.startswith("network_mode="):
                    file.write(f"network_mode={config.network_mode}\n")
                elif config.transmit_power and line.startswith("transmit_power="):
                    file.write(f"transmit_power={config.transmit_power}\n")
                elif config.encryption and line.startswith("encryption="):
                    file.write(f"encryption={config.encryption}\n")
                elif config.enabled is not None and line.startswith("enabled="):
                    file.write(f"enabled={'1' if config.enabled else '0'}\n")
                else:
                    file.write(line)

        if config.enabled is not None:
            subprocess.run(
                ["systemctl", "start" if config.enabled else "stop", "hostapd"]
            )
        else:
            subprocess.run(["systemctl", "restart", "hostapd"])

    @staticmethod
    def get_supported_frequencies() -> List[Dict[str, str]]:
        """
        Определяет поддерживаемые диапазоны частот Wi-Fi адаптера.
        Возвращает список доступных частот с диапазоном и мощностью передатчика.
        """
        try:
            output = subprocess.check_output(["iw", "list"]).decode()

            frequencies = []
            current_band = None
            for line in output.splitlines():
                line = line.strip()
                if line.startswith("Band 1"):
                    current_band = "2.4GHz"
                elif line.startswith("Band 2"):
                    current_band = "5GHz"
                elif line.startswith("Band 3"):
                    current_band = "6GHz"
                elif line.startswith("*") and "MHz" in line:
                    freq_match = re.search(r"(\d+) MHz", line)
                    power_match = re.search(r"(\d+\.?\d*) dBm", line)
                    if freq_match and power_match:
                        frequency = freq_match.group(1) + " MHz"
                        power = float(power_match.group(1))
                        frequencies.append(
                            {
                                "band": current_band,
                                "frequency": frequency,
                                "transmit_power": power,
                            }
                        )
            return frequencies
        except subprocess.CalledProcessError as e:
            print(f"Error reading supported frequencies: {e}")
            return []

    @staticmethod
    def update_wifi_status():
        """
        Проверяет и обновляет записи WiFiStatus в базе данных.
        Если записи отсутствуют, загружает поддерживаемые частоты и создает записи.
        """
        if not WiFiStatus.objects.exists():
            supported_frequencies = WifiManager.get_supported_frequencies()

            for freq_info in supported_frequencies:
                WiFiStatus.objects.create(
                    band=freq_info["band"],
                    frequency=freq_info["frequency"],
                    transmit_power=freq_info["transmit_power"],
                    enabled=False,
                )


class UpdateService:
    @staticmethod
    def check_updates():
        """
        Checks if updates are available.
        Returns True if updates are available, False otherwise.
        """
        try:
            result = subprocess.run(
                ["apt-get", "-s", "upgrade"],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                text=True,
            )
            if "0 upgraded, 0 newly installed" in result.stdout:
                return False
            return True
        except Exception as e:
            logger.error(f"Error checking updates: {str(e)}")
            raise

    @staticmethod
    def perform_update():
        """
        Performs the updates.
        Returns a tuple (success: bool, output: str).
        """
        try:
            result = subprocess.run(
                ["apt-get", "update", "-y"],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                text=True,
            )
            result_upgrade = subprocess.run(
                ["apt-get", "upgrade", "-y"],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                text=True,
            )
            if result_upgrade.returncode != 0:
                logger.error(f"Error during upgrade: {result_upgrade.stderr}")
                return False, result_upgrade.stderr
            return True, result_upgrade.stdout
        except Exception as e:
            logger.error(f"Error performing updates: {str(e)}")
            return False, str(e)